package com.jiuqi.util.inifile;

import com.jiuqi.util.Stream;
import java.util.ArrayList;
import java.util.List;

public class Section
{
    private String name;
    private List list;
    private IniBuffer buffer;
    private boolean founded;
    
    public Section(final String aName, final IniBuffer aBuffer) {
        this.name = aName;
        this.buffer = aBuffer;
        this.list = new ArrayList();
    }
    
    public String name() {
        return this.name;
    }
    
    public int count() {
        return this.list.size();
    }
    
    public Ident get(final int index) {
        return this.list.get(index);
    }
    
    public void readInfo() {
        this.clear();
        for (int l = this.buffer.readInfoInteger(), i = 0; i < l; ++i) {
            final Ident node = new Ident();
            node.name = this.buffer.readInfoString();
            node.type = this.buffer.readInfoByte();
            node.position = this.buffer.readInfoInteger();
            this.list.add(node);
            if (node.type == 7) {
                (node.child = new BufferIni(this.buffer)).readInfo();
            }
        }
    }
    
    public void writeInfo() {
        final int l = this.count();
        this.buffer.writeInfoInteger(l);
        for (int i = 0; i < l; ++i) {
            final Ident node = this.get(i);
            this.buffer.writeInfoString(node.name);
            this.buffer.writeInfoByte(node.type);
            this.buffer.writeInfoInteger(node.position);
            if (node.child != null) {
                node.child.writeInfo();
            }
        }
    }
    
    public void clear() {
        this.list.clear();
    }
    
    public void deleteKey(final int index) {
        this.list.remove(index);
    }
    
    public void deleteKey(final String ident) {
        final int i = this.find(ident);
        if (this.founded) {
            this.list.remove(i);
        }
    }
    
    public void writeNull(final String ident) {
        this.deleteKey(ident);
    }
    
    public void writeBoolean(final String ident, final boolean value) {
        final Ident node = this.findNodeCreate(ident);
        node.type = 1;
        node.child = null;
        node.position = this.buffer.writeBoolean(value);
    }
    
    public void writeDouble(final String ident, final double value) {
        final Ident node = this.findNodeCreate(ident);
        node.type = 3;
        node.child = null;
        node.position = this.buffer.writeDouble(value);
    }
    
    public void writeDateTime(final String ident, final long value) {
        this.writeLong(ident, value);
    }
    
    public void writeInteger(final String ident, final int value) {
        final Ident node = this.findNodeCreate(ident);
        node.type = 2;
        node.child = null;
        node.position = this.buffer.writeInteger(value);
    }
    
    public void writeLong(final String ident, final long value) {
        final Ident node = this.findNodeCreate(ident);
        node.type = 4;
        node.child = null;
        node.position = this.buffer.writeLong(value);
    }
    
    public void writeString(final String ident, final String value) {
        if (value == null) {
            this.deleteKey(ident);
        }
        else {
            final Ident node = this.findNodeCreate(ident);
            node.type = 5;
            node.child = null;
            node.position = this.buffer.writeString(value);
        }
    }
    
    public void writeStream(final String ident, final Stream stream) {
        try {
            final Ident node = this.findNodeCreate(ident);
            node.type = 6;
            node.child = null;
            final int l = (int)(stream.getSize() - stream.getPosition());
            node.position = this.buffer.writeInteger(l);
            this.buffer.writeStream(stream, l);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public BufferIni writeIni(final String ident) {
        final Ident node = this.findNodeCreate(ident);
        node.type = 7;
        if (node.child == null) {
            node.child = new BufferIni(this.buffer);
        }
        return node.child;
    }
    
    public int find(final String ident) {
        this.founded = false;
        int l = 0;
        int h = this.list.size() - 1;
        while (l <= h) {
            final int i = (l + h) / 2;
            final int c = this.get(i).name.compareTo(ident);
            if (c < 0) {
                l = i + 1;
            }
            else {
                h = i - 1;
                if (c != 0) {
                    continue;
                }
                this.founded = true;
                l = i;
            }
        }
        return l;
    }
    
    public Ident findNode(final String ident) {
        final int i = this.find(ident);
        if (this.founded) {
            return this.get(i);
        }
        return null;
    }
    
    public Ident findNodeCreate(final String ident) {
        final int i = this.find(ident);
        if (this.founded) {
            return this.get(i);
        }
        final Ident ret = new Ident();
        ret.name = ident;
        this.list.add(i, ret);
        return ret;
    }
    
    public boolean isNull(final String ident) {
        final Ident node = this.findNode(ident);
        return node == null;
    }
    
    public BufferIni readIni(final String ident) {
        final int i = this.find(ident);
        if (this.founded) {
            return this.get(i).child;
        }
        return null;
    }
    
    public boolean readBoolean(final String ident, final boolean defValue) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            return this.buffer.readBoolean(node.position);
        }
        return defValue;
    }
    
    public double readDouble(final String ident, final double defValue) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            return this.buffer.readDouble(node.position);
        }
        return defValue;
    }
    
    public long readDateTime(final String ident, final long defValue) {
        return this.readLong(ident, defValue);
    }
    
    public int readInteger(final String ident, final int defValue) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            return this.buffer.readInteger(node.position);
        }
        return defValue;
    }
    
    public long readLong(final String ident, final long defValue) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            return this.buffer.readLong(node.position);
        }
        return defValue;
    }
    
    public String readString(final String ident, final String defValue) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            return this.buffer.readString(node.position);
        }
        return defValue;
    }
    
    public void readStream(final String ident, final Stream stream) {
        final Ident node = this.findNode(ident);
        if (node != null) {
            try {
                final int l = this.buffer.readInteger(node.position);
                this.buffer.readStream(stream, node.position + 4, l);
            }
            catch (final Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}
